/********************************************************
 The %skmdata macro

 A SAS macro for the Kaplan-Meier plot with SGPLOT
 Procedure


 Copyright (c) 2015 Kengo NAGASHIMA

 This software is released under the MIT License,
 see <http://opensource.org/licenses/mit-license.php>.
********************************************************/

/********************************************************
 Author: Kengo NAGASHIMA
 Version: 1.0.2
 Last Updated Date: 04/22/2015
 System Requirements: SAS 9.4 or later, base SAS,
                      SAS/STAT 13.1 or later

 v1.0.2: fixed a bug which causes missing at risk sizes
 v1.0.1: fixed a bug which is caused by censored observations
 v1.0.0: first release
********************************************************/

/********************************************************
 Arguments
   data: input dataset name
   time: time variable
   event: event variable
   group: group variable
   censor_value: code value of censored observations
   out: output dataset name
   timemax: maximum time for no. at risk order
   timeby: increment of the sequence for no. at risk order
********************************************************/

%macro skmdata(data, time, event, group, censor_value, out, timemax, timeby);
%if "&group." = "" %then %do;
data _tmp_; set &data.; _group_ = 1;
%let group = _group_;
%end;
%else %do;
data _tmp_; set &data.;
%end;

ods graphics off;
proc lifetest data = &data. outsurv = _sdata_;
  time &time.*&event.(&censor_value.);
  strata &group.;
  ods output ProductLimitEstimates = _ple_;
run;
ods graphics on;
proc sort data = _sdata_; by STRATUM &time.;
data _sdata_;
  set _sdata_; by STRATUM &time.;
  retain est lcl ucl .;
  if SURVIVAL ^= . then est = SURVIVAL;
  else SURVIVAL = est;
  if SDF_LCL ^= . then lcl = SDF_LCL;
  else SDF_LCL = lcl;
  if SDF_UCL ^= . then ucl = SDF_UCL;
  else SDF_UCL = ucl;
  if last.STRATUM then do;
    est = .; lcl = .; ucl = .;
  end;
  if _censor_ = 1 then ycensor = SURVIVAL;
  else ycensor = .;
data _plex_; set _ple_;
proc sort data = _ple_; by STRATUM &time.;
data _ple_;
  set _ple_; by STRATUM &time.;
  do _x_ = 0 to &timemax. by &timeby.;
    if &time. = 0 & Censor = . then do; output; leave; end;
    else if &time. <= _x_ then do; output; leave; end;
  end;
data _ple_;
  format atrisk $5.;
  length atrisk $5.;
  set _ple_; by STRATUM _x_;
  if last._x_ then output;
data _pledummy_;
  set _ple_; by STRATUM;
  if first.STRATUM then do;
    do _x_ = 0 to &timemax. by &timeby.;
      output;
    end;
  end;
  keep STRATUM _x_ &group.;
data _ple_;
  merge _ple_ _pledummy_;
  retain tleft .;
  by STRATUM _x_;
  if left ^= . then tleft = left;
  else if left = . then left = tleft;
  if last.STRATUM then tleft = .;
  keep _x_ left &group.;
  rename _x_ = xatrisk left = atrisk;
run;
data &out.;
  set _sdata_ _ple_ _pledummy_;
proc datasets lib = work;
  delete _sdata_ _ple_;
run; quit;
%mend;
